# -*- coding: utf-8 -*-
from django.conf.urls import url
from django.shortcuts import render_to_response
from tastypie.authentication import SessionAuthentication
from tastypie.authorization import DjangoAuthorization
from tastypie.resources import ModelResource
import time
from bbs import defaults
from bbs.models import Topic, TopicComment
from bbs.resource.resource import BaseResource, TopicResource
from tastypie import fields
from django.template import RequestContext
from bbs.util import get_html_tag, remove_html_tag


class TopicCommentAuthorization(DjangoAuthorization):
    def create_list(self, object_list, bundle):
        return super(TopicCommentAuthorization, self).create_list(object_list, bundle).filter(user=bundle.request.user)

    def create_detail(self, object_list, bundle):
        is_auth = super(TopicCommentAuthorization, self).create_detail(object_list, bundle)
        return (bundle.obj.user == bundle.request.user) and is_auth

    def delete_list(self, object_list, bundle):
        return super(TopicCommentAuthorization, self).delete_list(object_list, bundle).filter(user=bundle.request.user)

    def delete_detail(self, object_list, bundle):
        is_auth = super(TopicCommentAuthorization, self).delete_detail(object_list, bundle)
        return (bundle.obj.user == bundle.request.user) and is_auth

    def update_list(self, object_list, bundle):
        return super(TopicCommentAuthorization, self).update_list(object_list, bundle).filter(user=bundle.request.user)

    def update_detail(self, object_list, bundle):
        is_auth = super(TopicCommentAuthorization, self).update_detail(object_list, bundle)
        return (bundle.obj.user == bundle.request.user) and is_auth

    def read_list(self, object_list, bundle):
        res = super(TopicCommentAuthorization, self).read_list(object_list, bundle)
        return res.filter(user=bundle.request.user)

    def read_detail(self, object_list, bundle):
        is_auth = super(TopicCommentAuthorization, self).read_detail(object_list, bundle)
        return (bundle.obj.user == bundle.request.user) and is_auth


class TopicCommentAuthentication(SessionAuthentication):
    def is_authenticated(self, request, **kwargs):
        is_auth = super(TopicCommentAuthentication, self).is_authenticated(request, **kwargs)
        return is_auth


class TopicCommentResource(BaseResource, ModelResource):
    topic = fields.ForeignKey(TopicResource, 'topic')

    def prepend_urls(self):
        return [
            url(r"^(?P<resource_name>%s)/comment-list/(?P<topic>\d+)%s$" % (self._meta.resource_name, defaults.URL_EXP),
                self.wrap_view('comment_list'), name='comment_list'),
            url(r"^(?P<resource_name>%s)/form/(?P<topic>\d+)%s$" % (self._meta.resource_name, defaults.URL_EXP),
                self.wrap_view('comment_form'), name='comment_form'),
        ]


    def comment_list(self, request, **kwargs):
        topic = Topic.objects.get(id=kwargs['topic'])
        if request.is_ajax():
            template = 'bbs/modal/topic_comment.html'
        else:
            template = 'bbs/pages/topic_comment.html'

        return render_to_response(template, {'topic': topic, 'topic_comment_set': topic.topiccomment_set.all()}, RequestContext(request))

    def obj_create(self, bundle, **kwargs):
        tags = list(get_html_tag(bundle.data['body']))
        bundle.data['body'] = remove_html_tag(bundle.data['body'], filter(lambda tag: not tag in ['span', 'strong'], tags))
        kwargs['user'] = bundle.request.user
        return super(TopicCommentResource, self).obj_create(bundle, **kwargs)

    def comment_form(self, request, **kwargs):
        topic = Topic.objects.get(id=kwargs['topic'])
        if request.is_ajax():
            template = 'bbs/modal/topic_comment_form.html'
        else:
            template = 'bbs/pages/topic_comment_form.html'
        return render_to_response(template, {'topic': topic, 'id': int(time.time())}, RequestContext(request))

    class Meta:
        queryset = TopicComment.objects.all()
        resource_name = 'topic-comment'
        authorization = TopicCommentAuthorization()
        authentication = TopicCommentAuthentication()
        list_allowed_methods = ['get', 'post']
        detail_allowed_methods = ['get', 'post']
        always_return_data = True