# -*- coding: utf-8 -*-
from django.conf.urls import url
from django.shortcuts import render_to_response
from tastypie.authentication import SessionAuthentication
from tastypie.authorization import DjangoAuthorization
from tastypie.resources import ModelResource
from bbs import defaults
from bbs.models import Topic, TopicScore
from bbs.resource.resource import BaseResource, TopicResource
from tastypie import fields
from django.template import RequestContext

class TopicScoreAuthorization(DjangoAuthorization):
    def create_list(self, object_list, bundle):
        return super(TopicScoreAuthorization, self).create_list(object_list, bundle).filter(user=bundle.request.user)

    def create_detail(self, object_list, bundle):
        is_auth = super(TopicScoreAuthorization, self).create_detail(object_list, bundle)
        return (bundle.obj.user == bundle.request.user) and is_auth

    def delete_list(self, object_list, bundle):
        return super(TopicScoreAuthorization, self).delete_list(object_list, bundle).filter(user=bundle.request.user)

    def delete_detail(self, object_list, bundle):
        is_auth = super(TopicScoreAuthorization, self).delete_detail(object_list, bundle)
        return (bundle.obj.user == bundle.request.user) and is_auth

    def update_list(self, object_list, bundle):
        return super(TopicScoreAuthorization, self).update_list(object_list, bundle).filter(user=bundle.request.user)

    def update_detail(self, object_list, bundle):
        is_auth = super(TopicScoreAuthorization, self).update_detail(object_list, bundle)
        return (bundle.obj.user == bundle.request.user) and is_auth

    def read_list(self, object_list, bundle):
        res = super(TopicScoreAuthorization, self).read_list(object_list, bundle)
        return res.filter(user=bundle.request.user)

    def read_detail(self, object_list, bundle):
        is_auth = super(TopicScoreAuthorization, self).read_detail(object_list, bundle)
        return (bundle.obj.user == bundle.request.user) and is_auth


class TopicScoreAuthentication(SessionAuthentication):
    def is_authenticated(self, request, **kwargs):
        is_auth = super(TopicScoreAuthentication, self).is_authenticated(request, **kwargs)
        return is_auth


class TopicScoreResource(BaseResource, ModelResource):
    topic = fields.ForeignKey(TopicResource, 'topic')

    def prepend_urls(self):
        return [
            url(r"^(?P<resource_name>%s)/score-list/(?P<topic>\d+)%s$" % (self._meta.resource_name, defaults.URL_EXP),
                self.wrap_view('score_list'), name='score_list'),
            url(r"^(?P<resource_name>%s)/form/(?P<topic>\d+)%s$" % (self._meta.resource_name, defaults.URL_EXP),
                self.wrap_view('score_form'), name='score_form'),
        ]

    def score_form(self, request, **kwargs):
        topic = Topic.objects.get(id=kwargs['topic'])
        if request.is_ajax():
            template = 'bbs/modal/topic_score_form.html'
        else:
            template = 'bbs/pages/topic_score_form.html'
        return render_to_response(template, {'topic': topic}, RequestContext(request))

    def score_list(self, request, **kwargs):
        topic = Topic.objects.get(id=kwargs['topic'])
        if request.is_ajax():
            template = 'bbs/modal/topic_score.html'
        else:
            template = 'bbs/pages/topic_score.html'
        return render_to_response(template, {'topic': topic, 'topic_score_set': topic.topicscore_set.all()}, RequestContext(request))

    def obj_create(self, bundle, **kwargs):
        kwargs['user'] = bundle.request.user
        return super(TopicScoreResource, self).obj_create(bundle, **kwargs)

    class Meta:
        queryset = TopicScore.objects.all()
        resource_name = 'topic-score'
        authorization = TopicScoreAuthorization()
        authentication = TopicScoreAuthentication()
        list_allowed_methods = ['get', 'post']
        detail_allowed_methods = ['get', 'post']
        always_return_data = True