# # -*- coding: utf-8 -*-
import os
import random
import shutil
from smtplib import SMTPRecipientsRefused
import uuid
from decimal import Decimal
from DjangoCaptcha import Captcha, Code
from django.conf import settings
from django.conf.urls import url
from django.contrib.auth import logout, authenticate, login
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse, NoReverseMatch
from django.db import transaction
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from tastypie import http
from tastypie.authentication import SessionAuthentication, Authentication
from tastypie.authorization import DjangoAuthorization
from tastypie.constants import ALL, ALL_WITH_RELATIONS
from tastypie.exceptions import ImmediateHttpResponse
from tastypie.resources import ModelResource, Resource
from tastypie.utils.urls import trailing_slash
import time
from bbs import defaults
from tastypie import fields
from bbs.exceptions import UserLoginError
from bbs.forms import SignupForm
from bbs.models import Forum, Category, Topic, TopicScore, Report


class BaseResource(ModelResource):
    def get_resource_uri(self, bundle_or_obj=None, url_name='bbs:api_dispatch_detail'):
        if bundle_or_obj is not None:
            url_name = 'bbs:api_dispatch_detail'

        try:
            return self._build_reverse_url(url_name, kwargs=self.resource_uri_kwargs(bundle_or_obj))
        except NoReverseMatch:
            return ''


class UserResource(BaseResource):
    def prepend_urls(self):
        return [
            url(r"^(?P<resource_name>%s)/check-username-exists%s$" % (self._meta.resource_name, trailing_slash()),
                self.wrap_view('check_username_exists'), name='check_username_exists'),
            url(r"^(?P<resource_name>%s)/check-email-exists%s$" % (self._meta.resource_name, trailing_slash()),
                self.wrap_view('check_email_exists'), name='check_email_exists'),
            url(r"^(?P<resource_name>%s)/check-email-exists%s$" % (self._meta.resource_name, trailing_slash()),
                self.wrap_view('check_email_exists'), name='check_email_exists'),
            url(r"^(?P<resource_name>%s)/signup%s$" % (self._meta.resource_name, trailing_slash()),
                self.wrap_view('signup'), name='signup_api'),
            url(r"^(?P<resource_name>%s)/signin%s$" % (self._meta.resource_name, trailing_slash()),
                self.wrap_view('signin'), name='signin_api'),
        ]



    # @transaction.atomic()
    def signup(self, request, **kwargs):
        self.method_check(request, allowed=['post'])
        data = self.deserialize(request, request.body,
                                format=request.META.get('CONTENT_TYPE', 'application/json'))
        signup_form = SignupForm(data)
        if signup_form.is_valid():
            ca = Captcha(request)
            if ca.check(data['captcha']):
                user = signup_form.save()
                if request.user.is_authenticated():
                    logout(request)
                user = authenticate(username=user.username, password=signup_form.cleaned_data['password'])
                login(request, user)
                return self.create_response(request, {"status": True,
                                                      'url': getattr(settings, 'LGOIN_REDIRECT',
                                                                     reverse('bbs:account'))})
            else:
                return self.create_response(request, {"status": False, 'content': '验证码错误'})
        else:
            return self.create_response(request, {"status": False, 'content': '请填写完整的信息.'})


    def signin(self, request, **kwargs):
        # 登陆错误次数,出现验证码
        login_error = int(request.session.get('login_error', 0))
        self.method_check(request, allowed=['post'])
        data = self.deserialize(request, request.body,
                                format=request.META.get('CONTENT_TYPE', 'application/json'))

        if defaults.IS_LOGIN_CAPTCHA(login_error):
            # 需要验证码
            code = Code(request)
            if not code.check(data.get('captcha')):
                return self.create_response(request, {"status": False, 'content': '验证码错误.', 'c': True})
        try:
            user = authenticate(identification=data['identification'], password=data['password'])
        except UserLoginError as e:
            login_error += 1
            request.session['login_error'] = login_error
            return self.create_response(request, {"status": False, 'content': e.message,
                                                  'c': login_error >= defaults.LOGIN_ERROR})

        if user:
            login(request, user)
            if request.session.get('login_error'):
                del request.session['login_error']
            if data.get('remember_me'):
                request.session.set_expiry(defaults.USERENA_REMEMBER_ME_DAYS)
            else:
                request.session.set_expiry(0)
            next = request.GET.get('next', getattr(settings, 'LGOIN_REDIRECT', reverse('bbs:account')))
            return self.create_response(request, {"status": True, 'url': next})
        else:
            login_error += 1
            request.session['login_error'] = login_error
            return self.create_response(request, {"status": False, 'content': '用户名或密码错误.',
                                                  'c': login_error >= defaults.LOGIN_ERROR})


    def check_username_exists(self, request, **kwargs):
        self.method_check(request, allowed=['post'])
        if User.objects.filter(username=request.POST.get('username')).exists():
            return self.create_response(request, {'valid': False, 'message': '用户名已经存在.'})
        else:
            return self.create_response(request, {'valid': True})

    def check_email_exists(self, request, **kwargs):
        self.method_check(request, allowed=['post'])
        if User.objects.filter(email=request.POST.get('email')).exists():
            return self.create_response(request, {'valid': False, 'message': '邮箱已经存在.'})
        else:
            return self.create_response(request, {'valid': True})

    def get_object_list(self, request):
        objects = super(UserResource, self).get_object_list(request).filter(id=request.user.id)
        return objects

    class Meta:
        queryset = User.objects.filter(pk__gt=0)
        fields = ['id', 'username', 'email', 'is_active']
        resource_name = 'user'
        authentication = SessionAuthentication()
        authorization = DjangoAuthorization()
        list_allowed_methods = ['get']
        detail_allowed_methods = ['get', 'post']
        always_return_data = True
        filtering = {
            'id': ALL,
        }


class CategoryResource(BaseResource):
    class Meta:
        queryset = Category.objects.all()
        resource_name = 'category'
        authorization = DjangoAuthorization()
        list_allowed_methods = ['get']
        detail_allowed_methods = ['get']
        always_return_data = True
        filtering = {
            'id': ALL,
        }


class ForumResource(BaseResource):
    category = fields.ForeignKey(CategoryResource, 'category', full=True)
    parent = fields.ForeignKey('self', 'parent', null=True, blank=True)

    def dehydrate(self, bundle):
        bundle.data["absolute_url"] = reverse("bbs:forum_detail", kwargs={'pk': bundle.obj.id})
        bundle.data["has_sub"] = bundle.obj.child_forums.exists()
        bundle.data["new_topic"] = reverse("bbs:topic_new", kwargs={'pk': bundle.obj.id, 'tab': 'topic'})
        return bundle

    class Meta:
        queryset = Forum.objects.all()
        resource_name = 'forum'
        authorization = DjangoAuthorization()
        list_allowed_methods = ['get']
        detail_allowed_methods = ['get']
        always_return_data = True
        filtering = {
            'id': ALL,
            'category': ALL_WITH_RELATIONS,
            'parent': ALL_WITH_RELATIONS,
        }


class TopicResource(BaseResource):
    def prepend_urls(self):
        return [
            url(r"^(?P<resource_name>%s)/reply%s$" % (self._meta.resource_name, trailing_slash()),
                self.wrap_view('reply'), name='topic_reply'),
        ]

    def reply(self, request, **kwargs):
        self.method_check(request, allowed=['get'])
        topic = Topic.objects.get(id=request.GET.get('id'))

        return self.create_response(request, {'body': topic.summary(), 'user': topic.user.username,
                                              'date': topic.created.strftime('%Y-%m-%d %H:%M'),
                                              'url': reverse('bbs:topic_goto', kwargs={'pk': topic.id})})

    class Meta:
        queryset = Topic.objects.all()
        resource_name = 'topic'
        authorization = DjangoAuthorization()
        list_allowed_methods = []
        detail_allowed_methods = []
        always_return_data = True


class ReportResource(BaseResource):

    def prepend_urls(self):
        return [
            url(r"^(?P<resource_name>%s)/form%s$" % (self._meta.resource_name, defaults.URL_EXP),
                self.wrap_view('report_form'), name='report_form'),
        ]

    def report_form(self, request, **kwargs):
        return render_to_response('bbs/modal/report_form.html', {}, RequestContext(request))

    class Meta:
        queryset = Report.objects.all()
        resource_name = 'report'
        authorization = DjangoAuthorization()
        authentication = SessionAuthentication()
        list_allowed_methods = []
        detail_allowed_methods = []
        always_return_data = True