# -*- coding: utf-8 -*-
import json
import os
import re
import urllib
import uuid
from django.utils.html import strip_tags
from HTMLParser import HTMLParser


def get_file_path(instance, filename, to):
    ext = filename.split('.')[-1]
    filename = "%s.%s" % (uuid.uuid4(), ext)
    return os.path.join(to, filename)


def summary(text, length):
    text = u''.join(strip_tags(text).split())

    if len(text) > length:
        text = '%s...' % text[0: length]
    return text


def short_url(url):
    response = urllib.urlopen('http://api.t.sina.com.cn/short_url/shorten.json', urllib.urlencode({
        'source': '3818214747',
        'url_long': url,
    }))
    body = response.read()
    response.close()
    return json.loads(body)[0]['url_short']


def validate_email(email):
    """ 校验邮件

    :param string email:
    :return bool:
    """
    from django.core.validators import validate_email
    from django.core.exceptions import ValidationError

    try:
        validate_email(email)
        return True
    except ValidationError:
        return False


def validate_phone(number):
    """ 校验手机

    :param string email:
    :return boolean:
    """

    return re.match(r'^0?1\d{10}$', number)


def validate_cardId(idcard):
    """ 校验身份证

    :param string 身份证号码:
    :return boolean:
    """
    if len(idcard) == 15:
        if (int(idcard[6:8]) + 1900) % 4 == 0 or (
                            (int(idcard[6:8]) + 1900) % 100 == 0 and (int(idcard[6:8]) + 1900) % 4 == 0 ):
            ereg = re.compile(
                '[1-9][0-9]{5}[0-9]{2}((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|[1-2][0-9]))[0-9]{3}$')  # //测试出生日期的合法性
        else:
            ereg = re.compile(
                '[1-9][0-9]{5}[0-9]{2}((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|1[0-9]|2[0-8]))[0-9]{3}$')  # //测试出生日期的合法性

        if re.match(ereg, idcard):
            return None
        else:
            return '身份证号码出生日期超出范围或含有非法字符.'
    elif len(idcard) == 18:
        # 出生日期的合法性检查
        # 闰年月日:((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|[1-2][0-9]))
        # 平年月日:((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|1[0-9]|2[0-8]))
        if int(idcard[6:10]) % 4 == 0 or (int(idcard[6:10]) % 100 == 0 and int(idcard[6:10]) % 4 == 0 ):
            ereg = re.compile(
                '[1-9][0-9]{5}19[0-9]{2}((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|[1-2][0-9]))[0-9]{3}[0-9Xx]$')  # //闰年出生日期的合法性正则表达式
        else:
            ereg = re.compile(
                '[1-9][0-9]{5}19[0-9]{2}((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|1[0-9]|2[0-8]))[0-9]{3}[0-9Xx]$')  # //平年出生日期的合法性正则表达式
            # //测试出生日期的合法性
        if re.match(ereg, idcard):
            return None
        else:
            return '身份证号码出生日期超出范围或含有非法字符.'
    return '身份证号码位数不对.'


def get_html_tag(html):
    tags = []

    class MyHTMLParser(HTMLParser):
        def handle_starttag(self, tag, attrs):
            tags.append(tag)

    p = MyHTMLParser()
    p.feed(html)
    return set(tags)


def remove_html_tag(html, tags):
    for tag in tags:
        re_data = re.compile('</?\s*?%s\s*?.*?/?>' % tag, re.I)
        html = re_data.sub('', html)
    return html
