# -*- coding: utf-8 -*-
from urllib import urlencode
from DjangoCaptcha import Captcha
from django.contrib.auth import logout
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import HttpResponse
from django.shortcuts import render, render_to_response
from django.shortcuts import render, redirect
from django.template import RequestContext
import math
from bbs import defaults
from bbs.auth.auth import _auth
from bbs.forms import TopicForm
from bbs.models import Category, Forum, Topic

# =======HOME==========
from django.core.paginator import Paginator


def home_view(request, template_name):
    template_context = {'t': range(0, 10)}

    return render_to_response(template_name, template_context, RequestContext(request))


# =======ACCOUNTS==========
def signup_view(request, template_name):
    template_context = {}
    return render_to_response(template_name, template_context, RequestContext(request))


def signin_view(request, template_name='bbs/signin.html'):
    template_context = {}
    request.breadcrumbs([("首页", reverse("bbs:home")),
                         ("登录", reverse("bbs:signin")),
    ])
    return render_to_response(template_name, template_context, RequestContext(request))


def signout_view(request):
    logout(request)
    try:
        url = request.META['HTTP_REFERER']
        return redirect(url)
    except:
        return redirect(reverse('bbs:home'))


def account_view(request, template_name):
    template_context = {}
    request.breadcrumbs([("首页", reverse("bbs:home")),
                         ("登录", reverse("bbs:signin")),
    ])

    return render_to_response(template_name, template_context, RequestContext(request))


def captcha_view(request):
    ca = Captcha(request)
    ca.img_width = 100
    ca.type = 'word'
    ca.words = 'a'

    return ca.display()


def forum_list_view(request, template_name, c=None):
    categories = Category.objects.all()

    request.breadcrumbs([("首页", reverse("bbs:home")),
                         ("论坛版块", reverse("bbs:forum")),

    ])
    if c:
        categories = categories.filter(id=c)
        request.breadcrumbs([("首页", reverse("bbs:home")),
                             ("论坛版块", reverse("bbs:forum")),
                             (categories[0].name, reverse("bbs:forum_category", kwargs={'c': c})),
        ])

    template_context = {'categories': categories}
    return render_to_response(template_name, template_context, RequestContext(request))


def forum_view(request, pk, template_name):
    forum = Forum.objects.get(id=pk)
    topics = forum.get_topics()

    template_context = {'forum': forum, 'topics': topics}

    request.breadcrumbs([("首页", reverse("bbs:home")),
                         (forum.category.name, reverse("bbs:forum_category", kwargs={'c': forum.category.id})),
                         (forum.name, reverse("bbs:forum_detail", kwargs={'pk': forum.id})),
    ])
    return render_to_response(template_name, template_context, RequestContext(request))


def topic_view(request, pk, template_name):
    topic = Topic.objects.get(id=pk, genus_topic__isnull=True)
    posts = topic.get_reply_set()
    template_context = {'topic': topic,
                        'forum': topic.forum,
                        'topic_score_set': topic.topicscore_set.all()[0:10],
                        'topic_comment_user_set': User.objects.filter(topiccomment__in=topic.topiccomment_set.all()).distinct(),
                        'topiccomment_set': topic.topiccomment_set.all()
    }
    qs = Q()
    if topic.ordertype:
        posts = posts.order_by('-created')
    if request.GET.get('see'):
        see = request.GET['see']
        if see == 'lz':
            see_user = topic.user
        else:
            see_user = User.objects.get(id=see)
        template_context['see_user'] = see_user
        qs = Q(qs & Q(user=see_user))
    posts = posts.filter(qs)
    template_context['posts'] = posts

    if request.method == 'POST':
        form = TopicForm(request.POST, request=request, topic=topic)
        template_context['form'] = form
        if form.is_valid():
            post = form.save()
            args = ''
            if request.POST.get('last'):
                if topic.ordertype:
                    args = "#t%s" % post.id
                else:
                    p = Paginator(posts, defaults.PAGE_SIZE)
                    args = "?page=%s#t%s" % (p.num_pages, post.id)
            return redirect(reverse('bbs:topic_detail', kwargs={'pk': post.genus_topic.id}) + args)

    request.breadcrumbs([("首页", reverse("bbs:home")),
                         (topic.forum.category.name,
                          reverse("bbs:forum_category", kwargs={'c': topic.forum.category.id})),
                         (topic.forum.name, reverse("bbs:forum_detail", kwargs={'pk': topic.forum.id})),
    ])
    return render_to_response(template_name, template_context, RequestContext(request))


def topic_goto(request, pk):
    topic = Topic.objects.get(id=pk)

    if not topic.genus_topic.ordertype:
        count = topic.genus_topic.topic_set.filter(created__lt=topic.created).count() + 1
    else:
        count = topic.genus_topic.topic_set.filter(created__gt=topic.created).count() + 1
    page = int(math.ceil(count / float(defaults.PAGE_SIZE)))
    args = {'page': page}
    for f in request.GET:
        args[f] = request.GET.get(f)
    return redirect("%s?%s#t%s"%(reverse('bbs:topic_detail', kwargs={'pk': topic.genus_topic.id}), urlencode(args), topic.id))

@_auth
def topic_new(request, tab, template_name, pk):
    template_context = {'tab': tab}
    if request.method == 'POST':
        form = TopicForm(request.POST, request=request)
        if form.is_valid():
            topic = form.save()
            return redirect(reverse('bbs:topic_detail', kwargs={"pk": topic.pk}))
        template_context['form'] = form

    if int(pk) != 0:
        forum = Forum.objects.get(id=pk)
        type_set = forum.topictype_set.all()
        template_context['forum'] = forum
        template_context['type_set'] = type_set
    else:
        template_name = 'bbs/topic_select_forum.html'
    request.breadcrumbs([("首页", reverse("bbs:home")),
                         ("发表帖子", reverse("bbs:topic_new", kwargs={'pk': pk, 'tab': tab})),

    ])
    return render_to_response(template_name, template_context, RequestContext(request))